# frozen_string_literal: true

module DockerCeCookbook
  module AptPackageHelper
    def build_docker_version
      expected_version = node[:docker_ce][:version]

      # Returning `nil` will install the latest available version
      return nil if expected_version == 'latest'

      detected = detect_docker_ce_version_from(expected_version)

      # Return the given version from the attributes when no version found
      expected_version if detected.nil? || detected.empty?

      # Returns the first detected Docker version
      puts "Using Docker package version #{detected} ..."
      detected
    end

    private

    # Gets all the available versions, makes a table of it, and returns only the
    # version column (Column with index 1)
    def available_docker_ce_versions
      @available_docker_ce_versions ||= begin
        versions = shell_out('apt-cache madison docker-ce').stdout.split("\n")
        versions.map { |version| version.split(' | ')[1] }
      end
    end

    #
    # Try to match, with a regex, a version and return it, otherwise show a
    # warning message and the available versions.
    #
    def detect_docker_ce_version_from(expected_version)
      versions = available_docker_ce_versions
      detected = versions.detect { |version| version =~ /#{expected_version}/ }

      unless detected
        puts 'WARNING: The Docker CE version you asked ' \
               "\"#{expected_version}\" couldn't be found, the install " \
               "command will fail. Here are the available version:\n\n" \
               "#{versions.join("\n")}"
      end

      detected
    end
  end
end
